@@ -1,4 +1,4 @@
-# Copyright (C) 2012, 2013, 2014 Julian Marchant <onpon4@riseup.net>
+# Copyright (C) 2012, 2013, 2014, 2015 Julian Marchant <onpon4@riseup.net>
 # 
 # This file is part of the Pygame SGE.
 # 
@@ -21,6 +21,7 @@ from __future__ import print_function
 from __future__ import unicode_literals
 
 import os
+import math
 import warnings
 
 import pygame
@@ -31,10 +32,10 @@ from sge import r
 from sge.r import (_scale, _get_blend_flags, _set_mode, _handle_music,
                    _get_dot_sprite, _get_line_sprite, _get_rectangle_sprite,
                    _get_ellipse_sprite, _get_circle_sprite,
-                   _get_polygon_sprite, _get_text_sprite, bl_update,
-                   bl_get_image, o_update, o_detect_collisions,
-                   o_get_image_width, o_get_normal_image_width,
-                   o_get_image_height, o_get_normal_image_height, s_get_image)
+                   _get_polygon_sprite, bl_update, bl_get_image, o_update,
+                   o_detect_collisions, o_get_image_width,
+                   o_get_normal_image_width, o_get_image_height,
+                   o_get_normal_image_height, s_get_image)
 
 
 __all__ = ['Game']
@@ -340,21 +341,11 @@ class Game(object):
 
     def start(self):
         """
-        Start the game at the first room.  Can be called in the middle
-        of a game to start the game over.  If you do this, everything
-        will be reset to its original state.
+        Start the game.  Should only be called once; the effect of any
+        further calls is undefined.
         """
-        if r.game_running:
-            self.alarms = {}
-
-            # Stop all sounds
-            sge.Music.stop()
-            sge.Sound.stop_all()
-
-            self.start_room.start()
-        elif self.start_room is not None:
+        if self.start_room is not None:
             r.game_running = True
-            self.event_game_start()
             self.start_room.start()
             r.game_clock.tick()
 
@@ -594,7 +585,6 @@ class Game(object):
                 # Refresh
                 self.refresh()
             else:
-                self.event_game_end()
                 pygame.quit()
                 sge.game = None
         else:
@@ -628,9 +618,8 @@ class Game(object):
             except IOError:
                 font = sge.Font("Droid Sans", size=24)
                 sprite = sge.Sprite(width=320, height=240)
-                sprite.draw_text(font, "Paused", 160, 120,
-                                 halign=sge.ALIGN_CENTER,
-                                 valign=sge.ALIGN_MIDDLE)
+                sprite.draw_text(font, "Paused", 160, 120, halign="center",
+                                 valign="middle")
 
         r.game_paused = True
 
@@ -732,7 +721,18 @@ class Game(object):
                     self.event_paused_close()
 
             # Regulate speed
-            self.regulate_speed()
+            time_passed = self.regulate_speed()
+
+            if self.delta:
+                t = min(time_passed, 1000 / self.delta_min)
+                delta_mult = t / (1000 / self.fps)
+            else:
+                delta_mult = 1
+
+            self.event_paused_step(time_passed, delta_mult)
+            self.current_room.event_paused_step(time_passed, delta_mult)
+            for obj in self.current_room.objects:
+                obj.event_paused_step(time_passed, delta_mult)
 
             # Project sprite
             x = (self.width - sprite.width) / 2
@@ -906,62 +906,52 @@ class Game(object):
         for view in self.current_room.views:
             view_surf = pygame.Surface((view.width, view.height))
             view_surf.fill(pygame.Color(*self.current_room.background.color))
+            vx = view.x - sge.game.current_room.background_x
+            vy = view.y - sge.game.current_room.background_y
 
             images = []
 
             for layer in self.current_room.background.layers:
                 img = bl_get_image(layer)
-                x = (sge.game.current_room.background_x + layer.x -
-                     (view.x * layer.xscroll_rate))
-                y = (sge.game.current_room.background_y + layer.y -
-                     (view.y * layer.yscroll_rate))
+                x = layer.x - (vx * layer.xscroll_rate)
+                y = layer.y - (vy * layer.yscroll_rate)
                 img_w = max(1, img.get_width())
                 img_h = max(1, img.get_height())
 
-                # These equations bring the position to the largest
-                # values possible while still being less than the
-                # location we're getting the surface at.  This is to
-                # minimize the number of repeat blittings.
-                if layer.xrepeat:
-                    x = (x % img_w) - img_w
-                if layer.yrepeat:
-                    y = (y % img_h) - img_h
-
                 # Apply the origin so the positions are as expected.
                 x -= layer.sprite.origin_x
                 y -= layer.sprite.origin_y
 
-                # Return x and y to being room relatives instead of view
-                # relatives.
-                x += view.x
-                y += view.y
+                # Move to the best position for what we want to do
+                if layer.repeat_right and (layer.repeat_left or x < 0):
+                    x = (x % img_w) - img_w
+                elif layer.repeat_left and x + img_w > view.width:
+                    x = (x % img_w) + img_w * math.ceil(view.width / img_w)
+                if layer.repeat_down and (layer.repeat_up or y < 0):
+                    y = (y % img_h) - img_h
+                elif layer.repeat_up and y + img_h > view.height:
+                    y = (y % img_h) + img_h * math.ceil(view.height / img_h)
 
-                xstart = x
-                ystart = y
+                if layer.repeat_right and (layer.repeat_left or x < view.width):
+                    hrange = six.moves.range(int(x),
+                                             int(view.width + img_w), img_w)
+                elif layer.repeat_left and x + img_w > 0:
+                    hrange = six.moves.range(int(x), -img_w, -img_w)
+                else:
+                    hrange = [x]
 
-                if layer.xrepeat and layer.yrepeat:
-                    for y in six.moves.range(int(round(ystart)),
-                                             int(round(view.y + view.height)),
-                                             img_h):
-                        for x in six.moves.range(
-                                int(round(xstart)),
-                                int(round(view.x + view.width)), img_w):
-                            images.append((img, x, y, layer.z, None))
-                elif (layer.xrepeat and y < view.y + view.height and
-                      y + img_h > view.y):
-                    for x in six.moves.range(int(round(xstart)),
-                                             int(round(view.x + view.width)),
-                                             img_w):
-                        images.append((img, x, y, layer.z, None))
-                elif (layer.yrepeat and x < view.x + view.width and
-                      x + img_w > view.x):
-                    for y in six.moves.range(int(round(ystart)),
-                                             int(round(view.y + view.height)),
-                                             img_h):
-                        images.append((img, x, y, layer.z, None))
-                elif (x < view.x + view.width and x + img_w > view.x and
-                      y < view.y + view.height and y + img_h > view.y):
-                    images.append((img, x, y, layer.z, None))
+                if layer.repeat_down and (layer.repeat_up or y < view.height):
+                    vrange = six.moves.range(int(y),
+                                             int(view.height + img_h), img_h)
+                elif layer.repeat_up and y + img_h > 0:
+                    vrange = six.moves.range(int(y), -img_h, -img_h)
+                else:
+                    vrange = [y]
+
+                for y in vrange:
+                    for x in hrange:
+                        images.append((img, x + view.x, y + view.y, layer.z,
+                                       None))
 
             for obj in self.current_room.objects:
                 if (obj.visible and obj.sprite is not None and
@@ -982,7 +972,6 @@ class Game(object):
                                    obj.z, None))
 
             images.extend(self.current_room.rd["projections"])
-            self.current_room.rd["projections"] = []
 
             images.sort(key=lambda img: img[3])
 
@@ -994,6 +983,8 @@ class Game(object):
                 _scale(view_surf, view.wport, view.hport),
                 (view.xport, view.yport))
 
+        self.current_room.rd["projections"] = []
+
         # Window projections
         self.mouse.project_cursor()
         for projection in r.game_window_projections:
@@ -1212,8 +1203,8 @@ class Game(object):
         r.game_window_projections.append((img, x, y, blend_mode))
 
     def project_text(self, font, text, x, y, width=None, height=None,
-                    color=sge.Color("black"), halign=sge.ALIGN_LEFT,
-                    valign=sge.ALIGN_TOP, anti_alias=True):
+                    color=sge.Color("black"), halign="left",
+                    valign="top", anti_alias=True):
         """
         Project text onto the game window.
 
@@ -1231,34 +1222,10 @@ class Game(object):
             e = "`{}` is not a sge.Color object.".format(repr(color))
             raise TypeError(e)
 
-        sprite = _get_text_sprite(font, text, width, height, color, halign,
-                                  valign, anti_alias)
-        if halign == sge.ALIGN_RIGHT:
-            x -= sprite.width
-        elif halign == sge.ALIGN_CENTER:
-            x -= sprite.width / 2
-        if valign == sge.ALIGN_BOTTOM:
-            y -= sprite.height
-        elif valign == sge.ALIGN_MIDDLE:
-            y -= sprite.height / 2
-
+        sprite = sge.Sprite.from_text(font, text, width, height, color, halign,
+                                      valign, anti_alias)
         self.project_sprite(sprite, 0, x, y)
 
-    def event_game_start(self):
-        """
-        Called when the game starts.  This is only called once (it is
-        not called again when the game restarts), and it is always the
-        very first event method called.
-        """
-        pass
-
-    def event_game_end(self):
-        """
-        Called when the game ends.  This is only called once, and it is
-        always the very last event method called.
-        """
-        pass
-
     def event_step(self, time_passed, delta_mult):
         """
         Called once each frame.
@@ -1277,7 +1244,7 @@ class Game(object):
         """
         Called when the value of an alarm reaches 0.
 
-        See the documentation for :meth:`sge.Game.alarms` for more
+        See the documentation for :attr:`sge.Game.alarms` for more
         information.
         """
         pass
@@ -1393,44 +1360,50 @@ class Game(object):
     def event_mouse_collision(self, other):
         """
         Proxy for :meth:`sge.game.mouse.event_collision`.  See the
-        documentation for :meth:`sge.StellarClass.event_collision` for
-        more information.
+        documentation for :meth:`sge.Object.event_collision` for more
+        information.
         """
         pass
 
     def event_mouse_collision_left(self, other):
         """
         Proxy for :meth:`sge.game.mouse.event_collision_left`.  See the
-        documentation for :meth:`sge.StellarClass.event_collision_left`
-        for more information.
+        documentation for :meth:`sge.Object.event_collision_left` for
+        more information.
         """
         self.event_mouse_collision(other)
 
     def event_mouse_collision_right(self, other):
         """
         Proxy for :meth:`sge.game.mouse.event_collision_right`.  See the
-        documentation for :meth:`sge.StellarClass.event_collision_right`
-        for more information.
+        documentation for :meth:`sge.Object.event_collision_right` for
+        more information.
         """
         self.event_mouse_collision(other)
 
     def event_mouse_collision_top(self, other):
         """
         Proxy for :meth:`sge.game.mouse.event_collision_top`.  See the
-        documentation for :meth:`sge.StellarClass.event_collision_top`
-        for more information.
+        documentation for :meth:`sge.Object.event_collision_top` for
+        more information.
         """
         self.event_mouse_collision(other)
 
     def event_mouse_collision_bottom(self, other):
         """
         Proxy for :meth:`sge.game.mouse.event_collision_bottom`.  See
-        the documentation for
-        :meth:`sge.StellarClass.event_collision_bottom` for more
-        information.
+        the documentation for :meth:`sge.Object.event_collision_bottom`
+        for more information.
         """
         self.event_mouse_collision(other)
 
+    def event_paused_step(self, time_passed, delta_mult):
+        """
+        See the documentation for :meth:`sge.Game.event_step` for more
+        information.
+        """
+        pass
+
     def event_paused_key_press(self, key, char):
         """
         See the documentation for :class:`sge.input.KeyPress` for more
