@@ -376,7 +376,13 @@ class Object(object):
 
     @bbox_x.setter
     def bbox_x(self, value):
-        self.__bbox_x = value
+        if value is not None:
+            self.__bbox_x = value
+        else:
+            if self.sprite is not None:
+                self.__bbox_x = self.sprite.bbox_x
+            else:
+                self.__bbox_x = 0
         o_update_collision_lists(self)
 
     @property
@@ -385,7 +391,13 @@ class Object(object):
 
     @bbox_y.setter
     def bbox_y(self, value):
-        self.__bbox_y = value
+        if value is not None:
+            self.__bbox_y = value
+        else:
+            if self.sprite is not None:
+                self.__bbox_y = self.sprite.bbox_y
+            else:
+                self.__bbox_y = 0
         o_update_collision_lists(self)
 
     @property
@@ -394,7 +406,13 @@ class Object(object):
 
     @bbox_width.setter
     def bbox_width(self, value):
-        self.__bbox_width = value
+        if value is not None:
+            self.__bbox_width = value
+        else:
+            if self.sprite is not None:
+                self.__bbox_width = self.sprite.bbox_width
+            else:
+                self.__bbox_width = 1
         o_update_collision_lists(self)
 
     @property
@@ -403,7 +421,13 @@ class Object(object):
 
     @bbox_height.setter
     def bbox_height(self, value):
-        self.__bbox_height = value
+        if value is not None:
+            self.__bbox_height = value
+        else:
+            if self.sprite is not None:
+                self.__bbox_height = self.sprite.bbox_height
+            else:
+                self.__bbox_height = 1
         o_update_collision_lists(self)
 
     @property
@@ -877,9 +901,7 @@ class Object(object):
 
     def event_end_step(self, time_passed, delta_mult):
         """
-        Called each frame after automatic updates to objects (such as
-        the effects of the speed variables), but before collision
-        events.
+        Called each frame after collision events.
 
         See the documentation for :meth:`sge.Game.event_step` for more
         information.
@@ -888,7 +910,7 @@ class Object(object):
 
     def event_alarm(self, alarm_id):
         """
-        See the documentation for :meth:`sge.Object.alarms` for more
+        See the documentation for :attr:`sge.Object.alarms` for more
         information.
         """
         pass
@@ -1172,6 +1194,13 @@ class Object(object):
         """
         pass
 
+    def event_paused_step(self, time_passed, delta_mult):
+        """
+        See the documentation for :meth:`sge.Game.event_step` for more
+        information.
+        """
+        pass
+
     def event_paused_key_press(self, key, char):
         """
         See the documentation for :class:`sge.input.KeyPress` for more
