@@ -1,4 +1,4 @@
-# Copyright (C) 2012, 2013, 2014 Julian Marchant <onpon4@riseup.net>
+# Copyright (C) 2012, 2013, 2014, 2015 Julian Marchant <onpon4@riseup.net>
 #
 # This file is part of the Pygame SGE.
 #
@@ -52,10 +52,11 @@ class Sprite(object):
 
        .. note::
 
-          Changing this attribute is a destructive transformation: it
-          can result in loss of pixel information, especially if it is
-          done repeatedly.  Because of this, it is advised that you do
-          not adjust this value for routine scaling.  Use the
+          Changing this attribute will cause the sprite to be scaled
+          horizontally.  This is a destructive transformation: it can
+          result in loss of pixel information, especially if it is done
+          repeatedly.  Because of this, it is advised that you do not
+          adjust this value for routine scaling.  Use the
           :attr:`image_xscale` attribute of a :class:`sge.Object`
           object instead.
 
@@ -65,10 +66,11 @@ class Sprite(object):
 
        .. note::
 
-          Changing this attribute is a destructive transformation: it
-          can result in loss of pixel information, especially if it is
-          done repeatedly.  Because of this, it is advised that you do
-          not adjust this value for routine scaling.  Use the
+          Changing this attribute will cause the sprite to be scaled
+          vertically.  This is a destructive transformation: it can
+          result in loss of pixel information, especially if it is done
+          repeatedly.  Because of this, it is advised that you do not
+          adjust this value for routine scaling.  Use the
           :attr:`image_yscale` attribute of a :class:`sge.Object`
           object instead.
 
@@ -199,16 +201,15 @@ class Sprite(object):
     def frames(self):
         return len(self.rd["baseimages"])
 
-    def __init__(self, name=None, width=None, height=None, transparent=True,
-                 origin_x=0, origin_y=0, fps=60, bbox_x=None, bbox_y=None,
-                 bbox_width=None, bbox_height=None):
+    def __init__(self, name=None, directory="", width=None, height=None,
+                 transparent=True, origin_x=0, origin_y=0, fps=60, bbox_x=None,
+                 bbox_y=None, bbox_width=None, bbox_height=None):
         """
         Arguments:
 
         - ``name`` -- The base name of the image files, used to find all
-          individual image files that make up the sprite's animation in
-          the paths specified in :data:`sge.image_directories`.  One of
-          the following rules will be used to find the images:
+          individual image files that make up the sprite's animation`.
+          One of the following rules will be used to find the images:
 
           - The base name plus a valid image extension.  If this rule is
             used, the image will be loaded as a single-frame sprite.
@@ -232,6 +233,8 @@ class Sprite(object):
           If none of the above rules can be used, :exc:`IOError` is
           raised.
 
+        - ``directory`` -- The directory to search for image files in.
+
         All other arguments set the respective initial attributes of the
         sprite.  See the documentation for :class:`Sprite` for more
         information.
@@ -243,48 +246,47 @@ class Sprite(object):
         self.rd["images"] = []
         self.rd["masks"] = {}
 
-        fname_single = None
+        fname_single = []
         fname_frames = []
-        fname_strip = None
+        fname_strip = []
         errlist = []
 
         if name is not None:
-            for path in sge.image_directories:
-                if os.path.isdir(path):
-                    fnames = os.listdir(path)
-                    for fname in fnames:
-                        full_fname = os.path.join(path, fname)
-                        if (fname.startswith(name) and
-                                os.path.isfile(full_fname)):
-                            root, ext = os.path.splitext(fname)
-                            if root == name:
-                                split = [name, '']
-                            elif root.rsplit('-', 1)[0] == name:
-                                split = root.rsplit('-', 1)
-                            elif root.rsplit('_', 1)[0] == name:
-                                split = root.rsplit('_', 1)
-                            else:
-                                continue
+            directory = os.path.abspath(directory)
+            for fname in os.listdir(directory):
+                full_fname = os.path.join(directory, fname)
+                if (fname.startswith(name) and
+                        os.path.isfile(full_fname)):
+                    root, ext = os.path.splitext(fname)
+                    if root == name:
+                        split = [name, '']
+                    elif root.rsplit('-', 1)[0] == name:
+                        split = root.rsplit('-', 1)
+                    elif root.rsplit('_', 1)[0] == name:
+                        split = root.rsplit('_', 1)
+                    else:
+                        continue
 
-                            if root == name:
-                                fname_single = full_fname
-                            elif split[1].isdigit():
-                                n = int(split[1])
-                                while len(fname_frames) - 1 < n:
-                                    fname_frames.append(None)
-                                fname_frames[n] = full_fname
-                            elif (split[1].startswith('strip') and
-                                  split[1][5:].isdigit()):
-                                fname_strip = full_fname
+                    if root == name:
+                        fname_single.append(full_fname)
+                    elif split[1].isdigit():
+                        n = int(split[1])
+                        while len(fname_frames) - 1 < n:
+                            fname_frames.append(None)
+                        fname_frames[n] = full_fname
+                    elif (split[1].startswith('strip') and
+                          split[1][5:].isdigit()):
+                        fname_strip.append(full_fname)
 
-            if fname_single:
+            if any(fname_single):
                 # Load the single image
-                try:
-                    img = pygame.image.load(fname_single)
-                except pygame.error as e:
-                    errlist.append(e)
-                else:
-                    self.rd["baseimages"].append(img)
+                for fname in fname_single:
+                    try:
+                        img = pygame.image.load(fname)
+                    except pygame.error as e:
+                        errlist.append(e)
+                    else:
+                        self.rd["baseimages"].append(img)
 
             if not self.rd["baseimages"] and any(fname_frames):
                 # Load the multiple images
@@ -297,43 +299,41 @@ class Sprite(object):
                         else:
                             self.rd["baseimages"].append(img)
 
-            if not self.rd["baseimages"] and fname_strip:
+            if not self.rd["baseimages"] and any(fname_strip):
                 # Load the strip (sprite sheet)
-                root, ext = os.path.splitext(os.path.basename(fname_strip))
-                assert '-' in root or '_' in root
-                assert (root.rsplit('-', 1)[0] == name or
-                        root.rsplit('_', 1)[0] == name)
-                if root.rsplit('-', 1)[0] == name:
-                    split = root.rsplit('-', 1)
-                else:
-                    split = root.rsplit('_', 1)
+                for fname in fname_strip:
+                    root, ext = os.path.splitext(os.path.basename(fname))
+                    assert '-' in root or '_' in root
+                    assert (root.rsplit('-', 1)[0] == name or
+                            root.rsplit('_', 1)[0] == name)
+                    if root.rsplit('-', 1)[0] == name:
+                        split = root.rsplit('-', 1)
+                    else:
+                        split = root.rsplit('_', 1)
 
-                try:
-                    sheet = pygame.image.load(fname_strip)
-                except pygame.error as e:
-                    errlist.append(e)
-                else:
-                    assert split[1][5:].isdigit()
-                    n = int(split[1][5:])
+                    try:
+                        sheet = pygame.image.load(fname)
+                    except pygame.error as e:
+                        errlist.append(e)
+                    else:
+                        assert split[1][5:].isdigit()
+                        n = int(split[1][5:])
 
-                    img_w = max(1, sheet.get_width()) // n
-                    img_h = max(1, sheet.get_height())
-                    for x in six.moves.range(0, img_w * n, img_w):
-                        rect = pygame.Rect(x, 0, img_w, img_h)
-                        img = sheet.subsurface(rect)
-                        self.rd["baseimages"].append(img)
+                        img_w = max(1, sheet.get_width()) // n
+                        img_h = max(1, sheet.get_height())
+                        for x in six.moves.range(0, img_w * n, img_w):
+                            rect = pygame.Rect(x, 0, img_w, img_h)
+                            img = sheet.subsurface(rect)
+                            self.rd["baseimages"].append(img)
 
             if not self.rd["baseimages"]:
-                print("Directories searched:")
-                for d in sge.image_directories:
-                    print(os.path.normpath(os.path.abspath(d)))
                 print("Pygame errors during search:")
                 if errlist:
                     for e in errlist:
                         print(e)
                 else:
                     print("None")
-                msg = 'Supported file(s) for sprite name "{}" not found.'.format(name)
+                msg = 'Supported file(s) for sprite name "{}" not found in {}'.format(name, directory)
                 raise IOError(msg)
         else:
             # Name is None; default to a blank rectangle.
@@ -851,8 +851,8 @@ class Sprite(object):
         s_refresh(self)
 
     def draw_text(self, font, text, x, y, width=None, height=None,
-                  color=sge.Color("black"), halign=sge.ALIGN_LEFT,
-                  valign=sge.ALIGN_TOP, anti_alias=True, frame=None):
+                  color=sge.Color("black"), halign="left", valign="top",
+                  anti_alias=True, frame=None):
         """
         Draw text on the sprite.
 
@@ -879,35 +879,35 @@ class Sprite(object):
           horizontal location of the origin of the imaginary rectangle
           the text is drawn in.  Can be set to one of the following:
 
-          - :data:`sge.ALIGN_LEFT` -- Align the text to the left of the
+          - ``"left"`` -- Align the text to the left of the imaginary
+            rectangle the text is drawn in.  Set the origin of the
+            imaginary rectangle to its left edge.
+          - ``"center"`` -- Align the text to the center of the
             imaginary rectangle the text is drawn in.  Set the origin of
-            the imaginary rectangle to its left edge.
-          - :data:`sge.ALIGN_CENTER` -- Align the text to the center of
-            the imaginary rectangle the text is drawn in.  Set the
-            origin of the imaginary rectangle to its center.
-          - :data:`sge.ALIGN_RIGHT` -- Align the text to the right of
-            the imaginary rectangle the text is drawn in.  Set the
-            origin of the imaginary rectangle to its right edge.
+            the imaginary rectangle to its center.
+          - ``"right"`` -- Align the text to the right of the imaginary
+            rectangle the text is drawn in.  Set the origin of the
+            imaginary rectangle to its right edge.
 
         - ``valign`` -- The vertical alignment of the text and the
           vertical location of the origin of the imaginary rectangle the
           text is drawn in.  Can be set to one of the following:
 
-          - :data:`sge.ALIGN_TOP` -- Align the text to the top of the
+          - ``"top"`` -- Align the text to the top of the imaginary
+            rectangle the text is drawn in.  Set the origin of the
+            imaginary rectangle to its top edge.  If the imaginary
+            rectangle is not tall enough to contain all of the text, cut
+            text off from the bottom.
+          - ``"middle"`` -- Align the the text to the middle of the
+            imaginary rectangle the text is drawn in.  Set the origin of
+            the imaginary rectangle to its middle.  If the imaginary
+            rectangle is not tall enough to contain all of the text, cut
+            text off equally from the top and bottom.
+          - ``"bottom"`` -- Align the text  to the bottom of the
             imaginary rectangle the text is drawn in.  Set the origin of
             the imaginary rectangle to its top edge.  If the imaginary
             rectangle is not tall enough to contain all of the text, cut
-            text off from the bottom.
-          - :data:`sge.ALIGN_MIDDLE` -- Align the the text to the middle
-            of the imaginary rectangle the text is drawn in.  Set the
-            origin of the imaginary rectangle to its middle.  If the
-            imaginary rectangle is not tall enough to contain all of the
-            text, cut text off equally from the top and bottom.
-          - :data:`sge.ALIGN_BOTTOM` -- Align the text  to the bottom of
-            the imaginary rectangle the text is drawn in.  Set the
-            origin of the imaginary rectangle to its top edge.  If the
-            imaginary rectangle is not tall enough to contain all of the
-            text, cut text off from the top.
+            text off from the top.
 
         - ``anti_alias`` -- Whether or not anti-aliasing should be used.
         - ``frame`` -- The frame of the sprite to draw on, where ``0``
@@ -939,38 +939,38 @@ class Sprite(object):
             rect = rendered_text.get_rect()
             rect.top = i * font.rd["font"].get_linesize()
 
-            if halign == sge.ALIGN_LEFT:
+            if halign.lower() == "left":
                 rect.left = text_rect.left
-            elif halign == sge.ALIGN_RIGHT:
+            elif halign.lower() == "right":
                 rect.right = text_rect.right
-            elif halign == sge.ALIGN_CENTER:
+            elif halign.lower() == "center":
                 rect.centerx = text_rect.centerx
 
             text_surf.blit(rendered_text, rect)
 
-        if valign == sge.ALIGN_TOP:
+        if valign.lower() == "top":
             text_rect.top = box_rect.top
-        elif valign == sge.ALIGN_BOTTOM:
+        elif valign.lower() == "bottom":
             text_rect.bottom = box_rect.bottom
-        elif valign == sge.ALIGN_MIDDLE:
+        elif valign.lower() == "middle":
             text_rect.centery = box_rect.centery
 
         box_surf.blit(text_surf, text_rect)
 
-        if halign == sge.ALIGN_LEFT:
+        if halign.lower() == "left":
             box_rect.left = x
-        elif halign == sge.ALIGN_RIGHT:
+        elif halign.lower() == "right":
             box_rect.right = x
-        elif halign == sge.ALIGN_CENTER:
+        elif halign.lower() == "center":
             box_rect.centerx = x
         else:
             box_rect.left = x
 
-        if valign == sge.ALIGN_TOP:
+        if valign.lower() == "top":
             box_rect.top = y
-        elif valign == sge.ALIGN_BOTTOM:
+        elif valign.lower() == "bottom":
             box_rect.bottom = y
-        elif valign == sge.ALIGN_MIDDLE:
+        elif valign.lower() == "middle":
             box_rect.centery = y
         else:
             box_rect.top = y
@@ -1058,6 +1058,82 @@ class Sprite(object):
             self.rd["locked"] = False
             s_refresh(self)
 
+    def mirror(self):
+        """Mirror the sprite horizontally."""
+        for i in six.moves.range(self.frames):
+            img = self.rd["baseimages"][i]
+            self.rd["baseimages"][i] = pygame.transform.flip(img, True, False)
+
+        s_refresh(self)
+
+    def flip(self):
+        """Flip the sprite vertically."""
+        for i in six.moves.range(self.frames):
+            img = self.rd["baseimages"][i]
+            self.rd["baseimages"][i] = pygame.transform.flip(img, False, True)
+
+        s_refresh(self)
+
+    def rotate(self, x, adaptive_resize=True):
+        """
+        Rotate the sprite about the center.
+
+        Arguments:
+
+        - ``x`` -- The rotation amount in degrees, with rotation in a
+          positive direction being counter-clockwise.
+        - ``adaptive_resize`` -- Whether or not the sprite should be
+          resized to accomodate rotation.  If this is :const:`True`,
+          rotation amounts other than multiples of 180 will result in
+          the size of the sprite being adapted to fit the whole rotated
+          image.  The origin will also be adjusted so that the rotation
+          is about the center.
+
+        .. note::
+
+           This is a destructive transformation: it can result in loss
+           of pixel information, especially if it is done repeatedly.
+           Because of this, it is advised that you do not adjust this
+           value for routine rotation.  Use the :attr:`image_rotation`
+           attribute of a :class:`sge.Object` object instead.
+        """
+        new_w = self.width
+        new_h = self.height
+        for i in six.moves.range(self.frames):
+            img = pygame.transform.rotate(self.rd["baseimages"][i], x)
+            new_w = img.get_width()
+            new_h = img.get_height()
+            if adaptive_resize:
+                self.rd["baseimages"][i] = img
+            else:
+                x = -(new_w - self.__w) / 2
+                y = -(new_h - self.__h) / 2
+                self.rd["baseimages"][i].fill(pygame.Color(0, 0, 0, 0))
+                self.rd["baseimages"][i].blit(img, (x, y))
+
+        if adaptive_resize:
+            self.origin_x += (new_w - self.__w) / 2
+            self.origin_y += (new_h - self.__h) / 2
+            self.__w = new_w
+            self.__h = new_h
+
+        s_refresh(self)
+
+    def copy(self):
+        """Return a copy of the sprite."""
+        new_copy = Sprite(width=self.width, height=self.height,
+                          transparent=self.transparent, origin_x=self.origin_x,
+                          origin_y=self.origin_y, fps=self.fps,
+                          bbox_x=self.bbox_x, bbox_y=self.bbox_y,
+                          bbox_width=self.bbox_width,
+                          bbox_height=self.bbox_height)
+        for i in range(1, self.frames):
+            new_copy.append_frame()
+        for i in range(self.frames):
+            new_copy.draw_sprite(self, i, self.origin_x, self.origin_y, i)
+
+        return new_copy
+
     def save(self, fname):
         """
         Save the sprite to an image file.
@@ -1090,7 +1166,34 @@ class Sprite(object):
             raise IOError(m)
 
     @classmethod
-    def from_tileset(cls, name, x=0, y=0, columns=1, rows=1, xsep=0, ysep=0,
+    def from_text(cls, font, text, width=None, height=None,
+                  color=sge.Color("black"), halign="left", valign="top",
+                  anti_alias=True):
+        """
+        Create a sprite, draw the given text on it, and return the
+        sprite.  See the documentation for :meth:`sge.Sprite.draw_text`
+        for more information.
+
+        The sprite's origin is set based on ``halign`` and ``valign``.
+        """
+        i = (cls, tuple(font.name), font.size, font.underline, font.bold,
+             font.italic, text, width, height, str(color), halign, valign,
+             anti_alias)
+        if i in r.cache.text_sprite:
+            s = r.cache.text_sprite[i]
+        else:
+            w = font.get_width(text, width, height)
+            h = font.get_height(text, width, height)
+            x = {"left": 0, "right": w, "center": w / 2}.get(halign.lower(), 0)
+            y = {"top": 0, "bottom": h, "middle": h / 2}.get(valign.lower(), 0)
+            s = cls(width=w, height=h, origin_x=x, origin_y=y)
+            s.draw_text(font, text, x, y, width, height, color, halign, valign)
+            r.cache.text_sprite[i] = s
+
+        return s
+
+    @classmethod
+    def from_tileset(cls, fname, x=0, y=0, columns=1, rows=1, xsep=0, ysep=0,
                      width=1, height=1, origin_x=0, origin_y=0,
                      transparent=True, fps=0, bbox_x=None, bbox_y=None,
                      bbox_width=None, bbox_height=None):
@@ -1099,9 +1202,8 @@ class Sprite(object):
 
         Arguments:
 
-        - ``name`` -- The base name of the image file containing the
-          tileset, not including the file extension, to be found in one
-          of the paths specified in :data:`sge.image_directories`.
+        - ``fname`` -- The path to the image file containing the
+          tileset.
         - ``x`` -- The horizontal location relative to the image of the
           top-leftmost tile in the tileset.
         - ``y`` -- The vertical location relative to the image of the
@@ -1120,26 +1222,28 @@ class Sprite(object):
         sprite, ordered first from left to right and then from top to
         bottom.
         """
-        self = cls(name, width=width, height=height, origin_x=origin_x,
+        self = cls(width=width, height=height, origin_x=origin_x,
                    origin_y=origin_y, transparent=transparent, fps=fps,
                    bbox_x=bbox_x, bbox_y=bbox_y, bbox_width=bbox_width,
                    bbox_height=bbox_height)
-        tileset = Sprite(name, origin_x=x, origin_y=y)
 
-        for i in six.moves.range(1, rows):
-            for j in six.moves.range(1, columns):
-                self.append_frame()
+        try:
+            tileset = pygame.image.load(fname)
+        except pygame.error as e:
+            raise IOError(e)
 
-        self.draw_clear()
+        for i in six.moves.range(1, rows * columns):
+            self.append_frame()
 
         for i in six.moves.range(rows):
             for j in six.moves.range(columns):
                 frame = i * columns + j
-                x_ = (width + xsep) * j
-                y_ = (height + ysep) * i
-                self.draw_sprite(tileset, 0, -x_ - self.origin_x,
-                                 -y_ - self.origin_y, frame=frame)
+                x_ = x + self.origin_x + (width + xsep) * j
+                y_ = y + self.origin_y + (height + ysep) * i
+                self.rd["baseimages"][frame].blit(
+                    s_set_transparency(self, tileset), (-x_, -y_))
 
+        s_refresh(self)
         return self
 
     @classmethod
@@ -1174,3 +1278,9 @@ class Sprite(object):
         sprite.rd["baseimages"][0].blit(r.game_display_surface, (-x, -y))
         s_refresh(sprite)
         return sprite
+
+    def __copy__(self):
+        return self.copy()
+
+    def __deepcopy__(self, memo):
+        return self.copy()
