@@ -1,4 +1,4 @@
-# Copyright (C) 2012, 2013, 2014 Julian Marchant <onpon4@riseup.net>
+# Copyright (C) 2012, 2013, 2014, 2015 Julian Marchant <onpon4@riseup.net>
 #
 # This file is part of the Pygame SGE.
 #
@@ -61,7 +61,7 @@ numbers are to the left of the origin.  However, in the vertical
 direction, normally called ``y``, ``0`` is the origin, positive numbers
 are below the origin, and negative numbers are above the origin.  While
 slightly jarring if you are used to normal graphs, this is in fact
-common in 2-D game development and is so how pixels in most image
+common in 2-D game development and is also how pixels in most image
 formats are indexed.
 
 Except where otherwise specified, the origin is always located at the
@@ -134,30 +134,6 @@ Global Variables and Constants
 
    A string indicating the name of the SGE implementation.
 
-.. data:: sge.ALIGN_LEFT
-
-   Flag indicating horizontal alignment to the left.
-
-.. data:: sge.ALIGN_CENTER
-
-   Flag indicating horizontal alignment to the center.
-
-.. data:: sge.ALIGN_RIGHT
-
-   Flag indicating horizontal alignment to the right.
-
-.. data:: sge.ALIGN_TOP
-
-   Flag indicating vertical alignment to the top
-
-.. data:: sge.ALIGN_MIDDLE
-
-   Flag indicating vertical alignment to the middle.
-
-.. data:: sge.ALIGN_BOTTOM
-
-   Flag indicating vertical alignment to the bottom.
-
 .. data:: sge.BLEND_NORMAL
 
    Flag indicating normal blending.
@@ -337,7 +313,7 @@ from __future__ import absolute_import
 from __future__ import print_function
 from __future__ import unicode_literals
 
-__version__ = "0.15"
+__version__ = "0.16"
 
 import sys
 import os
@@ -348,36 +324,21 @@ import pygame
 IMPLEMENTATION = "Pygame SGE"
 COLLISION_AREA_SIZE_DEFAULT = 128
 
-ALIGN_LEFT = 2
-ALIGN_CENTER = 3
-ALIGN_RIGHT = 1
-ALIGN_TOP = 8
-ALIGN_MIDDLE = 12
-ALIGN_BOTTOM = 4
+BLEND_NORMAL = None
+BLEND_ALPHA = 1
+BLEND_RGB_ADD = 2
+BLEND_RGB_SUBTRACT = 4
+BLEND_RGB_MULTIPLY = 6
+BLEND_RGB_SCREEN = 8
+BLEND_RGB_MINIMUM = 10
+BLEND_RGB_MAXIMUM = 12
 
-BLEND_NORMAL = 0
-BLEND_RGB = 1
-BLEND_ALPHA = 2
-BLEND_ADD = 4
-BLEND_SUBTRACT = 8
-BLEND_MULTIPLY = 12
-BLEND_SCREEN = 16
-BLEND_MINIMUM = 20
-BLEND_MAXIMUM = 24
-
-BLEND_RGBA_ADD = BLEND_RGB | BLEND_ALPHA | BLEND_ADD
-BLEND_RGBA_SUBTRACT = BLEND_RGB | BLEND_ALPHA | BLEND_SUBTRACT
-BLEND_RGBA_MULTIPLY = BLEND_RGB | BLEND_ALPHA | BLEND_MULTIPLY
-BLEND_RGBA_SCREEN = BLEND_RGB | BLEND_ALPHA | BLEND_SCREEN
-BLEND_RGBA_MINIMUM = BLEND_RGB | BLEND_ALPHA | BLEND_MINIMUM
-BLEND_RGBA_MAXIMUM = BLEND_RGB | BLEND_ALPHA | BLEND_MAXIMUM
-
-BLEND_RGB_ADD = BLEND_RGB | BLEND_ADD
-BLEND_RGB_SUBTRACT = BLEND_RGB | BLEND_SUBTRACT
-BLEND_RGB_MULTIPLY = BLEND_RGB | BLEND_MULTIPLY
-BLEND_RGB_SCREEN = BLEND_RGB | BLEND_SCREEN
-BLEND_RGB_MINIMUM = BLEND_RGB | BLEND_MINIMUM
-BLEND_RGB_MAXIMUM = BLEND_RGB | BLEND_MAXIMUM
+BLEND_RGBA_ADD = BLEND_ALPHA | BLEND_RGB_ADD
+BLEND_RGBA_SUBTRACT = BLEND_ALPHA | BLEND_RGB_SUBTRACT
+BLEND_RGBA_MULTIPLY = BLEND_ALPHA | BLEND_RGB_MULTIPLY
+BLEND_RGBA_SCREEN = BLEND_ALPHA | BLEND_RGB_SCREEN
+BLEND_RGBA_MINIMUM = BLEND_ALPHA | BLEND_RGB_MINIMUM
+BLEND_RGBA_MAXIMUM = BLEND_ALPHA | BLEND_RGB_MAXIMUM
 
 MUSIC_END_EVENT = pygame.USEREVENT + 1
 
@@ -471,8 +432,7 @@ __all__ = [
     "collision", "input", "joystick", "keyboard", "mouse",
 
     # Constants
-    'IMPLEMENTATION', 'ALIGN_LEFT', 'ALIGN_CENTER', 'ALIGN_RIGHT', 'ALIGN_TOP',
-    'ALIGN_MIDDLE', 'ALIGN_BOTTOM', 'BLEND_RGBA_ADD', 'BLEND_RGBA_SUBTRACT',
+    'IMPLEMENTATION', 'BLEND_RGBA_ADD', 'BLEND_RGBA_SUBTRACT',
     'BLEND_RGBA_MULTIPLY', 'BLEND_RGBA_SCREEN', 'BLEND_RGBA_MINIMUM',
     'BLEND_RGBA_MAXIMUM', 'BLEND_RGB_ADD', 'BLEND_RGB_SUBTRACT',
     'BLEND_RGB_MULTIPLY', 'BLEND_RGB_SCREEN', 'BLEND_RGB_MINIMUM',
