@@ -1,4 +1,4 @@
-# Copyright (C) 2012, 2013, 2014 Julian Marchant <onpon4@riseup.net>
+# Copyright (C) 2012, 2013, 2014, 2015 Julian Marchant <onpon4@riseup.net>
 #
 # This file is part of the Pygame SGE.
 #
@@ -80,7 +80,8 @@ def _scale(surface, width, height):
 
 def _get_blend_flags(blend_mode):
     # Return the appropriate Pygame flags for the given blend mode.
-    if blend_mode is not None and blend_mode & sge.BLEND_SCREEN:
+    if blend_mode is not None and blend_mode in (sge.BLEND_RGB_SCREEN,
+                                                 sge.BLEND_RGBA_SCREEN):
         w = "Screen blend mode not supported. Normal blending used instead."
         warnings.warn(w)
 
@@ -345,31 +346,6 @@ def _get_polygon_sprite(points, fill, outline, outline_thickness, anti_alias):
     return sprite
 
 
-def _get_text_sprite(font, text, width, height, color, halign, valign,
-                     anti_alias):
-    # Return a sprite for the given text.
-    i = (font, text, width, height, tuple(color), halign, valign,
-         anti_alias)
-    if i in cache.text_sprite:
-        sprite = cache.text_sprite[i]
-    else:
-        if not isinstance(font, sge.Font):
-            font = sge.game.fonts[font]
-
-        w = font.get_width(text, width, height)
-        h = font.get_height(text, width, height)
-        draw_x = {sge.ALIGN_LEFT: 0, sge.ALIGN_CENTER: w / 2,
-                  sge.ALIGN_RIGHT: w}.setdefault(halign, w / 2)
-        draw_y = {sge.ALIGN_TOP: 0, sge.ALIGN_MIDDLE: h / 2,
-                  sge.ALIGN_BOTTOM: h}.setdefault(valign, h / 2)
-        sprite = sge.Sprite(None, width=w, height=h)
-        sprite.draw_text(font, text, draw_x, draw_y, width, height, color,
-                         halign, valign, anti_alias)
-        cache.text_sprite[i] = sprite
-
-    return sprite
-
-
 def _get_rectangle_collision_areas(x, y, w, h):
     # Get a list of collision areas a rect is in.
     room = sge.game.current_room
@@ -703,45 +679,6 @@ def o_set_speed(self):
     self.rd["mv_dir"] = math.degrees(math.atan2(-self.rd["yv"], self.rd["xv"]))
 
 
-def r_reset(self):
-    # Reset the room to its original state.
-    self.rd["started"] = False
-    self.width = self.rd["swidth"]
-    self.height = self.rd["sheight"]
-    self.views = self.rd["sviews"]
-    self.background = self.rd["sbackground"]
-    self.background_x = self.rd["sbackground_x"]
-    self.background_y = self.rd["sbackground_y"]
-    self.objects = self.rd["sobjects"][:]
-    self.alarms = {}
-
-    for view in self.views:
-        view.x = self.rd["view_sx"][id(view)]
-        view.y = self.rd["view_sy"][id(view)]
-        view.xport = self.rd["view_sxport"][id(view)]
-        view.yport = self.rd["view_syport"][id(view)]
-        view.width = self.rd["view_swidth"][id(view)]
-        view.height = self.rd["view_sheight"][id(view)]
-
-    for obj in self.objects:
-        if obj is not sge.game.mouse:
-            obj.x = self.rd["object_sx"][id(obj)]
-            obj.y = self.rd["object_sy"][id(obj)]
-
-        obj.z = self.rd["object_sz"][id(obj)]
-        obj.sprite = self.rd["object_ssprite"][id(obj)]
-        obj.visible = self.rd["object_svisible"][id(obj)]
-        obj.checks_collisions = self.rd["object_schecks_collisions"][id(obj)]
-        obj.tangible = self.rd["object_stangible"][id(obj)]
-        obj.bbox_x = self.rd["object_sbbox_x"][id(obj)]
-        obj.bbox_y = self.rd["object_sbbox_y"][id(obj)]
-        obj.bbox_width = self.rd["object_sbbox_width"][id(obj)]
-        obj.bbox_height = self.rd["object_sbbox_height"][id(obj)]
-        obj.collision_ellipse = self.rd["object_scollision_ellipse"][id(obj)]
-        obj.collision_precise = self.rd["object_scollision_precise"][id(obj)]
-        obj.alarms = {}
-
-
 def r_update_fade(self, complete):
     transition_sprite = self.rd["t_sprite"]
     w = transition_sprite.width
@@ -943,8 +880,7 @@ def s_set_size(self):
     # Adjust the size of the base images.  Note: this change is
     # destructive and irreversible.  It is necessary for the drawing
     # methods to work properly, specifically whenever ``width`` and
-    # ``height`` are set.  As a result, setting ``width`` and
-    # ``height`` is destructive in this implementation.
+    # ``height`` are set.
     width = int(round(self.width))
     height = int(round(self.height))
     for i in six.moves.range(self.frames):
